@@ -64,8 +64,7 @@ CVBooster <- R6::R6Class(
 #' dtrain <- gpb.Dataset(X, label = y)
 #' params <- list(learning_rate = 0.05,
 #'                max_depth = 6,
-#'                min_data_in_leaf = 5,
-#'                objective = "regression_l2")
+#'                min_data_in_leaf = 5)
 #' # Run CV
 #' cvbst <- gpb.cv(params = params,
 #'                 data = dtrain,
@@ -497,14 +496,16 @@ gpb.cv <- function(params = list()
                                           , free_raw_data = TRUE)
         valid_set_gp <- NULL
         if (use_gp_model_for_validation) {
-          gp_model_train$set_prediction_data(group_data_pred = group_data_pred,
-                                             group_rand_coef_data_pred = group_rand_coef_data_pred,
-                                             gp_coords_pred = gp_coords_pred,
-                                             gp_rand_coef_data_pred = gp_rand_coef_data_pred,
-                                             cluster_ids_pred = cluster_ids_pred,
-                                             vecchia_pred_type = gp_model$.__enclos_env__$private$vecchia_pred_type,
-                                             num_neighbors_pred = gp_model$.__enclos_env__$private$num_neighbors_pred,
-                                             cg_delta_conv_pred = gp_model$.__enclos_env__$private$cg_delta_conv_pred)
+          gp_model_train$set_prediction_data(group_data_pred = group_data_pred
+                                             , group_rand_coef_data_pred = group_rand_coef_data_pred
+                                             , gp_coords_pred = gp_coords_pred
+                                             , gp_rand_coef_data_pred = gp_rand_coef_data_pred
+                                             , cluster_ids_pred = cluster_ids_pred
+                                             , vecchia_pred_type = gp_model$.__enclos_env__$private$vecchia_pred_type
+                                             , num_neighbors_pred = gp_model$.__enclos_env__$private$num_neighbors_pred
+                                             , cg_delta_conv_pred = gp_model$.__enclos_env__$private$cg_delta_conv_pred
+                                             , nsim_var_pred = gp_model$.__enclos_env__$private$nsim_var_pred
+                                             , rank_pred_approx_matrix_lanczos = gp_model$.__enclos_env__$private$rank_pred_approx_matrix_lanczos)
           if (has_custom_eval_functions) {
             # Note: Validation using the GP model is only done in R if there are custom evaluation functions in eval_functions, 
             #        otherwise it is directly done in C++. See the function Eval() in regression_metric.hpp
@@ -516,10 +517,11 @@ gpb.cv <- function(params = list()
           }
           
         }
+        
         booster <- Booster$new(params = params, train_set = dtrain, gp_model = gp_model_train)
         gp_model$set_likelihood(gp_model_train$get_likelihood_name()) ## potentially change likelihood in case this was done in the booster to reflect implied changes in the default optimizer for different likelihoods
         gp_model_train$set_optim_params(params = gp_model$get_optim_params())
-        
+
       } else {
         booster <- Booster$new(params = params, train_set = dtrain)
       }
@@ -673,7 +675,13 @@ generate.cv.folds <- function(nfold, nrows, stratified, label, group, params) {
     rnd_idx <- sample.int(nrows)
     
     # Request stratified folds
-    if (isTRUE(stratified) && params$objective %in% c("binary", "multiclass") && length(label) == length(rnd_idx)) {
+    stratified_folds <- FALSE
+    if (!is.null(params$objective)) {
+      if (isTRUE(stratified) && params$objective %in% c("binary", "multiclass") && length(label) == length(rnd_idx)) {
+        stratified_folds <- TRUE
+      }
+    }
+    if (stratified_folds) {
       
       y <- label[rnd_idx]
       y <- as.factor(y)
@@ -914,36 +922,47 @@ get.param.combination <- function(param_comb_number, param_grid) {
 #' library(gpboost)
 #' data(GPBoost_data, package = "gpboost")
 #' 
-#' # Create random effects model, dataset, and define parameter grif
+#' # Create random effects model, dataset, and define parameter grid
 #' gp_model <- GPModel(group_data = group_data[,1], likelihood="gaussian")
-#' dtrain <- gpb.Dataset(X, label = y)
-#' params <- list(objective = "regression_l2")
-#' param_grid = list("learning_rate" = c(0.1,0.01), "min_data_in_leaf" = c(20),
-#'                   "max_depth" = c(5,10), "num_leaves" = 2^17, "max_bin" = c(255,1000))
-#' # Parameter tuning using cross-validation and deterministic grid search
+#' dataset <- gpb.Dataset(X, label = y)
+#' param_grid = list("learning_rate" = c(1,0.1,0.01), 
+#'                   "min_data_in_leaf" = c(10,100,1000),
+#'                   "max_depth" = c(1,2,3,5,10),
+#'                   "lambda_l2" = c(0,1,10))
+#' other_params <- list(num_leaves = 2^10)
+#' # Note: here we try different values for 'max_depth' and thus set 'num_leaves' to a large value.
+#' #       An alternative strategy is to impose no limit on 'max_depth', 
+#' #       and try different values for 'num_leaves' as follows:
+#' # param_grid = list("learning_rate" = c(1,0.1,0.01), 
+#' #                   "min_data_in_leaf" = c(10,100,1000),
+#' #                   "num_leaves" = 2^(1:10),
+#' #                   "lambda_l2" = c(0,1,10))
+#' # other_params <- list(max_depth = -1)
 #' set.seed(1)
-#' opt_params <- gpb.grid.search.tune.parameters(param_grid = param_grid,
-#'                                               params = params,
-#'                                               num_try_random = NULL,
-#'                                               nfold = 4,
-#'                                               data = dtrain,
-#'                                               gp_model = gp_model,
-#'                                               verbose_eval = 1,
-#'                                               nrounds = 1000,
-#'                                               early_stopping_rounds = 5,
-#'                                               eval = "l2")
-#' # Parameter tuning using cross-validation and random grid search
-#' set.seed(1)
-#' opt_params <- gpb.grid.search.tune.parameters(param_grid = param_grid,
-#'                                               params = params,
-#'                                               num_try_random = 4,
-#'                                               nfold = 4,
-#'                                               data = dtrain,
-#'                                               gp_model = gp_model,
-#'                                               verbose_eval = 1,
-#'                                               nrounds = 1000,
-#'                                               early_stopping_rounds = 5,
-#'                                               eval = "l2")
+#' opt_params <- gpb.grid.search.tune.parameters(param_grid = param_grid, params = other_params,
+#'                                               num_try_random = NULL, nfold = 4,
+#'                                               data = dataset, gp_model = gp_model,
+#'                                               use_gp_model_for_validation=TRUE, verbose_eval = 1,
+#'                                               nrounds = 1000, early_stopping_rounds = 10)
+#' print(paste0("Best parameters: ",
+#'              paste0(unlist(lapply(seq_along(opt_params$best_params), 
+#'                                   function(y, n, i) { paste0(n[[i]],": ", y[[i]]) }, 
+#'                                   y=opt_params$best_params, 
+#'                                   n=names(opt_params$best_params))), collapse=", ")))
+#' print(paste0("Best number of iterations: ", opt_params$best_iter))
+#' print(paste0("Best score: ", round(opt_params$best_score, digits=3)))
+#' # Note: other scoring / evaluation metrics can be chosen using the 
+#' #       'metric' argument, e.g., metric = "l1"
+#' 
+#' # Using manually defined validation data instead of cross-validation
+#' valid_tune_idx <- sample.int(length(y), as.integer(0.2*length(y)))
+#' folds = list(valid_tune_idx)
+#' opt_params <- gpb.grid.search.tune.parameters(param_grid = param_grid, params = other_params,
+#'                                               num_try_random = NULL, folds = folds,
+#'                                               data = dataset, gp_model = gp_model,
+#'                                               use_gp_model_for_validation=TRUE, verbose_eval = 1,
+#'                                               nrounds = 1000, early_stopping_rounds = 10)
+#' 
 #' }
 #' @author Fabio Sigrist
 #' @export
@@ -1008,15 +1027,15 @@ gpb.grid.search.tune.parameters <- function(param_grid
   if (is.null(num_try_random)) {
     try_param_combs <- 0:(grid_size - 1L)
     if (verbose_eval >= 1L) {
-      message(paste0("gpb.grid.search.tune.parameters: Starting deterministic grid search with ", grid_size, " parameter combinations..."))
+      message(paste0("gpb.grid.search.tune.parameters: Starting deterministic grid search with ", grid_size, " parameter combinations "))
     }
   } else {
     if(num_try_random > grid_size){
-      stop("gpb.grid.search.tune.parameters: num_try_random is larger than the number of all possible combinations of parameters in param_grid")
+      stop("gpb.grid.search.tune.parameters: num_try_random is larger than the number of all possible combinations of parameters in param_grid ")
     }
     try_param_combs <- sample.int(n = grid_size, size = num_try_random, replace = FALSE) - 1L
     if (verbose_eval >= 1L) {
-      message(paste0("gpb.grid.search.tune.parameters: Starting random grid search with ", num_try_random, " trials out of ", grid_size, " parameter combinations..."))
+      message(paste0("gpb.grid.search.tune.parameters: Starting random grid search with ", num_try_random, " trials out of ", grid_size, " parameter combinations "))
     }
   }
   if (verbose_eval < 2) {
@@ -1035,62 +1054,71 @@ gpb.grid.search.tune.parameters <- function(param_grid
     for (param in names(param_comb)) {
       params[[param]] <- param_comb[[param]]
     }
+    param_comb_str <- lapply(seq_along(param_comb), function(y, n, i) { paste0(n[[i]],": ", y[[i]]) }, y=param_comb, n=names(param_comb))
+    param_comb_str <- paste0(unlist(param_comb_str), collapse=", ")
     if (verbose_eval >= 1L) {
-      param_comb_str <- lapply(seq_along(param_comb), function(y, n, i) { paste0(n[[i]],": ", y[[i]]) }, y=param_comb, n=names(param_comb))
-      param_comb_str <- paste0(unlist(param_comb_str), collapse=", ")
       message(paste0("Trying parameter combination ", counter_num_comb, 
-                     " of ", length(try_param_combs), ": ", param_comb_str, " ..."))
+                     " of ", length(try_param_combs), ": ", param_comb_str))
     }
-    cvbst <- gpb.cv(params = params
-                    , data = data
-                    , nrounds = nrounds
-                    , gp_model = gp_model
-                    , use_gp_model_for_validation = use_gp_model_for_validation
-                    , train_gp_model_cov_pars = train_gp_model_cov_pars
-                    , folds = folds
-                    , nfold = nfold
-                    , label = label
-                    , weight = weight
-                    , obj = obj
-                    , eval = eval
-                    , verbose = verbose_cv
-                    , record = TRUE
-                    , eval_freq = 1L
-                    , showsd = FALSE
-                    , stratified = stratified
-                    , init_model = init_model
-                    , colnames = colnames
-                    , categorical_feature = categorical_feature
-                    , early_stopping_rounds = early_stopping_rounds
-                    , callbacks = callbacks
-                    , delete_boosters_folds = TRUE
-                    , ...
-    )
     current_score_is_better <- FALSE
-    if (higher_better) {
-      if (cvbst$best_score > best_score) {
-        current_score_is_better <- TRUE
+    tryCatch(
+      {
+        cvbst <- gpb.cv(params = params
+                        , data = data
+                        , nrounds = nrounds
+                        , gp_model = gp_model
+                        , use_gp_model_for_validation = use_gp_model_for_validation
+                        , train_gp_model_cov_pars = train_gp_model_cov_pars
+                        , folds = folds
+                        , nfold = nfold
+                        , label = label
+                        , weight = weight
+                        , obj = obj
+                        , eval = eval
+                        , verbose = verbose_cv
+                        , record = TRUE
+                        , eval_freq = 1L
+                        , showsd = FALSE
+                        , stratified = stratified
+                        , init_model = init_model
+                        , colnames = colnames
+                        , categorical_feature = categorical_feature
+                        , early_stopping_rounds = early_stopping_rounds
+                        , callbacks = callbacks
+                        , delete_boosters_folds = TRUE
+                        , ...
+        )
+        if (higher_better) {
+          if (cvbst$best_score > best_score) {
+            current_score_is_better <- TRUE
+          }
+        } else {
+          if (cvbst$best_score < best_score) {
+            current_score_is_better <- TRUE
+          }
+        }
+      },
+      error = function(msg) {
+        message(paste0("Error for parameter combination ", counter_num_comb, 
+                       " of ", length(try_param_combs), ": ", param_comb_str,
+                       ". Error message: "))
+        message(msg)
+      })# end tryCatch
+      if (current_score_is_better) {
+        best_score <- cvbst$best_score
+        best_iter <- cvbst$best_iter
+        best_params <- param_comb
+        if (verbose_eval >= 1L) {
+          metric_name <- names(cvbst$record_evals$valid)
+          param_comb_print <- param_comb
+          param_comb_print[["nrounds"]] <- best_iter
+          str <- lapply(seq_along(param_comb_print), function(y, n, i) { paste0(n[[i]],": ", y[[i]]) }, y=param_comb_print, n=names(param_comb_print))
+          message(paste0("***** New best test score (",metric_name, " = ", 
+                         best_score,  ") found for the following parameter combination: ", 
+                         paste0(unlist(str), collapse=", ")))
+        }
       }
-    } else {
-      if (cvbst$best_score < best_score) {
-        current_score_is_better <- TRUE
-      }
-    }
-    if (current_score_is_better) {
-      best_score <- cvbst$best_score
-      best_iter <- cvbst$best_iter
-      best_params <- param_comb
-      if (verbose_eval >= 1L) {
-        metric_name <- names(cvbst$record_evals$valid)
-        param_comb_print <- param_comb
-        param_comb_print[["nrounds"]] <- best_iter
-        str <- lapply(seq_along(param_comb_print), function(y, n, i) { paste0(n[[i]],": ", y[[i]]) }, y=param_comb_print, n=names(param_comb_print))
-        message(paste0("***** New best test score (",metric_name, " = ", 
-                       best_score,  ") found for the following parameter combination: ", 
-                       paste0(unlist(str), collapse=", ")))
-      }
-    }
-    counter_num_comb <- counter_num_comb + 1L
+      counter_num_comb <- counter_num_comb + 1L
   }
   
   return(list(best_params=best_params, best_iter=best_iter, best_score=best_score))
